<?php

/**
 * @file
 * Contains Batch API callbacks used during installation.
 */

/**
 * BatchAPI callback.
 */
function _dvg_enable_module($module, $module_name, &$context) {
  module_enable(array($module), FALSE);
  $context['message'] = st('Installed %module module', array('%module' => $module_name));
}

/**
 * BatchAPI callback.
 */
function _dvg_uninstall_module($module, &$context) {
  module_disable(array($module), FALSE);
  drupal_uninstall_modules(array($module));
  $context['message'] = st('Uninstalled %module module.', array('%module' => $module));
}

/**
 * BatchAPI callback.
 */
function _dvg_flush_caches($operation, &$context) {
  $context['message'] = t('@operation', array('@operation' => $operation));
  drupal_flush_all_caches();
}


/**
 * BatchAPI callback.
 */
function _dvg_install_set_default_settings($operation, &$context) {
  $context['message'] = t('@operation', array('@operation' => $operation));
  variable_set('i18n_string_source_language', 'en');
  variable_set('date_first_day', 1); // First day of week: monday.

  // Add new date formats and use theme for locale 'nl'.
  db_query("REPLACE INTO {date_formats} (format, type, locked) VALUES ('d-m-Y H:i', 'custom', '0')");
  db_query("REPLACE INTO {date_formats} (format, type, locked) VALUES ('D, d-m-Y - H:i', 'custom', '0')");
  db_query("REPLACE INTO {date_format_locale} (format, type, language) VALUES ('d-m-Y H:i', 'short', 'nl')");
  db_query("REPLACE INTO {date_format_locale} (format, type, language) VALUES ('D, d/m/Y - H:i', 'medium', 'nl')");
  db_query("REPLACE INTO {date_format_locale} (format, type, language) VALUES ('l, j. F Y - G:i', 'long', 'nl')");
}

/**
 * BatchAPI callback.
 */
function _dvg_install_enable_dvg_theme($operation, &$context) {
  $context['message'] = t('@operation', array('@operation' => $operation));
  theme_disable(array('bartik'));
  theme_enable(array('dvg'));
  variable_set('theme_default', 'dvg');
  variable_set('admin_theme', 'seven');
  variable_set('node_admin_theme', TRUE);
}

/**
 * BatchAPI callback.
 */
function _dvg_install_disable_file_types($operation, &$context) {
  $context['message'] = t('@operation', array('@operation' => $operation));
  if (function_exists('file_type_disable')) {
    $file_types = array('image', 'video', 'audio', 'document');
    foreach ($file_types as $file_type) {
      file_type_disable($file_type);
    }
  }
}

/**
 * BatchAPI callback.
 */
function _dvg_install_import_dvg_translations($operation, &$context) {
  $context['message'] = t('@operation', array('@operation' => $operation));
  module_load_install('dvg_global');
  _dvg_global_import_translations(__DIR__ . '/translations/' . basename(__FILE__, '.profile') . '.nl.po');
}

/**
 * BatchAPI callback.
 */
function _dvg_install_permissions($operation, &$context) {
  $context['message'] = t('@operation', array('@operation' => $operation));

  $permissions = array(
    'administrator' => array(
      'access administration menu',
    ),
    'super editor' => array(
      'access administration menu',
    ),
    'editor' => array(
      'access administration menu',
    ),
  );

  // Set permissions.
  foreach (user_roles(FALSE) as $rid => $role) {
    if (!empty($permissions[$role])) {
      user_role_grant_permissions($rid, $permissions[$role]);
    }
  }
}

/**
 * BatchAPI callback.
 */
function _dvg_install_revert_all_features($operation, &$context) {
  $context['message'] = t('@operation', array('@operation' => $operation));
  features_rebuild();
  features_revert();
}

/**
 * BatchAPI callback.
 */
function _dvg_menu_import_file($menu, &$context) {
  $context['message'] = st('Installed menu %menu', array('%menu' => $menu));
  $file = __DIR__ . '/demo-content/' . $menu . '-export.txt';
  menu_import_file($file, $menu, array(
    'link_to_content' => TRUE,
    'create_content' => FALSE,
    'remove_menu_items' => FALSE,
  ));
}

/**
 * BatchAPI callback.
 *
 * @see: node_export_import();
 */
function _dvg_import_node($original_node, &$context) {
  $context['message'] = st('Imported %title', array('%title' => $original_node->title));

  $used_format = 'drupal';
  $save = TRUE;
  $node = node_export_node_clone($original_node);
  $nodes = array($node);

  drupal_alter('node_export_import', $nodes, $used_format, $save);
  node_export_file_field_import($nodes[0], $original_node);
  node_export_save($nodes[0]);
  drupal_alter('node_export_node_import', $node, $original_node, $save);
}
