<?php

/**
 * Implements Ctools form wizard callback().
 */
function appointments_delete_form_wizard($step = NULL) {
  // Make sure the page isn't cached
  drupal_page_is_cacheable(FALSE);

  // Include CTools form wizard and CTools object cache.
  ctools_include('wizard');
  ctools_include('object-cache');
  module_load_include('inc', 'appointments', 'appointments_qmatic_api');

  // Button labels match the next trail item labels.
  $next_button_labels = array(
    NULL => t('Control'),
    'start' => t('Control'),
    'second' => t('Delete'),
  );
  $next_text = isset($next_button_labels[$step]) ? $next_button_labels[$step] : t('Next');

  // Configure ctools form wizard
  $form_info = array(
    'id' => 'appointments_delete',
    'path' => "appointments/delete/%step",
    //Show or not the progress of the passage of steps
    'show trail' => TRUE,
    // Turning on/off extra buttons
    'show back' => TRUE,
    'show cancel' => FALSE,
    'show return' => FALSE,
    // Texts for buttons
    'next text' => $next_text,
    'back text' => '< ' . t('previous step'),
    // Callbacks for specials buttons
    'next callback' => 'appointments_delete_wizard_next',
    'back callback' => 'appointments_delete_wizard_back',
    'finish callback' => 'appointments_delete_wizard_finish',
    // Array where we define a steps name.
    'order' => array(
      'start' => t('Appointment'),
      'second' => t('Control'),
      'third' => t('Confirmation'),
    ),
    // Array where we define a forms.
    'forms' => array(
      'start' => array(
        'form id' => 'appointments_delete_step1',
      ),
      'second' => array(
        'form id' => 'appointments_delete_step2',
      ),
      'third' => array(
        'form id' => 'appointments_delete_step3',
      ),
    ),
  );

  $object_id = 'appointments_delete';
  if (empty($step)) {
    // We reset the form when $step is NULL because that means they have
    // for whatever reason started over.
    appointments_cache_clear($object_id);
    $step = 'start';
  }

  // This automatically gets defaults if there wasn't anything saved.
  $object = appointments_cache_get($object_id);

  $form_state = array(
    'object_id' => $object_id,
    'object' => &$object,
  );

  $output = ctools_wizard_multistep_form($form_info, $step, $form_state);

  return $output;
}

/**
 * Handles the 'next' click on the add/edit pane form wizard.
 *
 * All we need to do is store the updated pane in the cache.
 */
function appointments_delete_wizard_next(&$form_state) {
  if ($form_state['values']['op'] != 'Back') {
    appointments_cache_set($form_state['object_id'], $form_state['object']);
  }
}


/**
 * Handles the 'finish' click on teh add/edit pane form wizard.
 *
 * All we need to do is set a flag so the return can handle adding
 * the pane.
 */
function appointments_delete_wizard_finish(&$form_state) {
  $form_state['complete'] = TRUE;
}

/**
 * Handles the 'cancel' click on the add/edit pane form wizard.
 */
function appointments_delete_wizard_cancel(&$form_state) {
  $form_state['cancel'] = TRUE;
}

/*
 * Generete First form.
 *
 * @param array $form
 *   Form array.
 *
 * @param $form_state
 *   All date for the form.
 */
function appointments_delete_step1($form, &$form_state) {
  $form['appointment_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Appointment ID'),
    '#size' => 8,
    '#maxlength' => 128,
    '#required' => TRUE,
    '#default_value' => isset($form_state['object']->appointment->appointmentId) ? $form_state['object']->appointment->appointmentId : '',
  );

  $form['email'] = array(
    '#type' => 'textfield',
    '#title' => t('E-mail'),
    '#required' => TRUE,
    '#default_value' => isset($form_state['object']->email) ? $form_state['object']->email : '',
  );

  return $form;
}

function appointments_delete_step1_validate($form, &$form_state) {
  // Validate email.
  if (!valid_email_address($form_state['values']['email'])) {
    form_set_error('email', t('The e-mail address you specified is not valid.'));
    return;
  }

  $qmatic = new appointments_qmatic_client();
  $now = new DateTime();
  $now = $now->format('Y-m-d\TH:i:s\Z');
  $appointments = $qmatic->get_customer_appointments(md5($form_state['values']['email']), $now);

  if (!$appointments) {
    form_set_error('appointment_id', t('There are no appointment with this ID or the entered email is not correct.'));
  }
  else {
    if (is_array($appointments)) {
      $found = FALSE;
      foreach ($appointments as $appointment) {
        if ($appointment->appointmentId == $form_state['values']['appointment_id']) {
          $form_state['object']->appointment = $appointment;
          $found = TRUE;
        }
      }
      if (!$found) {
        form_set_error('appointment_id', t('There is no appointment with this id.'));
      }
    }
    else {
      if ($appointments->appointmentId != $form_state['values']['appointment_id']) {
        form_set_error('appointment_id', t('There is no appointment with this id'));
      }
      else {
        $form_state['object']->appointment = $appointments;
      }
    }
  }
}

function appointments_delete_step1_submit($form, &$form_state) {
  $form_state['object']->email = $form_state['values']['email'];
}

/*
 * Generete First form.
 *
 * @param array $form
 *   Form array.
 *
 * @param $form_state
 *   All date for the form.
 */
function appointments_delete_step2($form, &$form_state) {
  // Redirect to step 1 in case no appointment data is available at this point.
  if(!isset($form_state['object']->appointment)) {
    drupal_goto('appointments/delete/start');
  }

  _appointments_fix_previous_button($form);

  $appointment = $form_state['object']->appointment;
  $appointment_info = array();

  $appointment_info[] = array(
    'label' => t('Subject'),
    'value' => $appointment->productName,
  );

  $appointment_info[] = array(
    'label' => t('Number of persons'),
    'value' => '1',
  );

  $date = strtotime($appointment->appointmentDate);
  $start_time = strtotime($appointment->appointmentTime);
  $end_time = $start_time + ($appointment->appointmentLength * 60);

  $appointment_info[] = array(
    'label' => t('Date'),
    'value' => appointments_format_date($date, 'dvg_short_date'),
  );

  $appointment_info[] = array(
    'label' => t('Time'),
    'value' => t('@start till @end', array('@start' => appointments_format_date($start_time, 'time'), '@end' => appointments_format_date($end_time, 'time'))),
  );

  $output_info = theme('appointments_info_table', $appointment_info);

  $form['appointment_info'] = array(
    '#markup' => $output_info,
  );

  return $form;
}

function appointments_delete_step2_submit($form, &$form_state) {
  $error = FALSE;
  $qmatic = new appointments_qmatic_client();
  if (isset($form_state['object']->appointment->appointmentId) && $response = $qmatic->delete_appointment($form_state['object']->appointment->appointmentId)) {
    if ($response->appointmentStatus != 1) {
      $error = TRUE;
    }
  }
  else {
    $error = TRUE;
  }

  if ($error) {
    drupal_set_message(t('An unknown error occurred. Please, try again.'), 'error');
    $url = str_replace($form_state['step'], 'start', $_REQUEST['q']);
    drupal_goto($url);
  }

  $form_state['object']->appointment_is_removed = TRUE;
  $date = strtotime($form_state['object']->appointment->appointmentDate);
  $duration = $form_state['object']->appointment->appointmentLength;
  $start_time = strtotime($form_state['object']->appointment->appointmentTime);
  $end_time = $start_time + ($duration * 60);

  $mail_array = array(
    '%id' => $form_state['object']->appointment->appointmentId,
    '%subject' => $form_state['object']->appointment->productName,
    '%date' => appointments_format_date($date, 'dvg_short_date'),
    '%start_time' => appointments_format_date($start_time, 'time'),
    '%end_time' => appointments_format_date($end_time, 'time'),
    '%email' => $form_state['object']->email,
    '%estimated_duration' => $form_state['object']->appointment->appointmentLength,
    '%create_appointment_link' => url('afspraak', array('absolute' => TRUE)),
  );

  appointments_cancellation_mail_send($form_state['object']->email, $mail_array);
  $second_email = variable_get('appointments_second_email', '');
  if (!empty($second_email)){
    appointments_cancellation_mail_send($second_email, $mail_array);
  }
}

/*
 * Generete First form.
 *
 * @param array $form
 *   Form array.
 *
 * @param $form_state
 *   All date for the form.
 */
function appointments_delete_step3($form, &$form_state) {
  // Redirect to step 1 in case no appointment data is available at this point.
  if(!isset($form_state['object']->appointment)) {
    drupal_goto('appointments/delete/start');
  }

  if (isset($form['buttons']['previous']) && isset($form['buttons']['return'])) {
    unset($form['buttons']);
  }

  $form['header'] = array(
    '#markup' => '<h2>' . t('Your appointment has been cancelled') . '</h2>',
  );

  // Display an error message in case the appointment couldn't be removed.
  $appointment_is_removed = isset($form_state['object']->appointment_is_removed) ? $form_state['object']->appointment_is_removed : FALSE;
  if (!$appointment_is_removed) {
    drupal_set_message(t('Some error happened, please try again!'), 'error');
  }

  $appointment = $form_state['object']->appointment;

  $appointment_info = array();

  $appointment_info[] = array(
    'label' => t('Subject'),
    'value' => $appointment->productName,
  );

  $appointment_info[] = array(
    'label' => t('Number of persons'),
    'value' => '1',
  );

  $date = strtotime($appointment->appointmentDate);
  $start_time = strtotime($appointment->appointmentTime);
  $end_time = $start_time + ($appointment->appointmentLength * 60);

  $appointment_info[] = array(
    'label' => t('Date'),
    'value' => appointments_format_date($date, 'dvg_short_date'),
  );

  $appointment_info[] = array(
    'label' => t('Time'),
    'value' => t('@start till @end', array('@start' => appointments_format_date($start_time, 'time'), '@end' => appointments_format_date($end_time, 'time'))),
  );

  $appointment_info[] = array(
    'label' => t('Appointment id'),
    'value' => $appointment->appointmentId,
  );

  $output_info = theme('appointments_info_table', $appointment_info);

  $form['appointment_info'] = array(
    '#markup' => $output_info,
  );


  $form['new_appointment'] = array(
    '#markup' => l(t('Add new appointment'), 'appointments/register/', array('attributes'=> array('class' => 'new-appointment'))),
  );

  return $form;
}
