<?php

/**
 * Implements Ctools form wizard callback().
 */
function appointments_registration_form_wizard($step = NULL) {
  // Make sure the page isn't cached
  drupal_page_is_cacheable(FALSE);

  // Include CTools form wizard and CTools object cache.
  ctools_include('wizard');
  ctools_include('object-cache');
  module_load_include('inc', 'appointments', 'appointments_qmatic_api');

  // Button labels match the next trail item labels.
  $next_button_labels = array(
    NULL => t('Date and time'),
    'start' => t('Date and time'),
    'second' => t('Your details'),
    'third' => t('Control'),
    'fourth' => t('Confirmation'),
  );
  $next_text = isset($next_button_labels[$step]) ? $next_button_labels[$step] : t('Next');

  // Configure ctools wizard
  $form_info = array(
    'id' => 'appointments',
    'path' => "appointments/register/%step",
    // Whether or not the progress of the passage of steps
    'show trail' => TRUE,
    //Show or not additional buttons
    'show back' => TRUE,
    'show cancel' => FALSE,
    'show return' => FALSE,
    // Text for buttons
    'next text' => $next_text,
    'back text' => '< ' . t('previous step'),
    'next callback' => 'appointments_wizard_next',
    'back callback' => 'appointments_wizard_back',
    'finish callback' => 'appointments_wizard_finish',
    'order' => array(
      'start' => t('Subject'),
      'second' => t('Date and time'),
      'third' => t('Your details'),
      'fourth' => t('Control'),
      'fifth' => t('Confirmation'),
    ),
    'forms' => array(
      'start' => array(
        'form id' => 'appointments_step1',
      ),
      'second' => array(
        'form id' => 'appointments_step2',
      ),
      'third' => array(
        'form id' => 'appointments_step3',
      ),
      'fourth' => array(
        'form id' => 'appointments_step4',
      ),
      'fifth' => array(
        'form id' => 'appointments_step5',
      ),
    ),
  );

  $object_id = 'appointments';
  if (empty($step)) {

    // We reset the form when $step is NULL because that means they have
    // for whatever reason started over.
    appointments_cache_clear($object_id);
    $step = 'start';
  }

  if ($step == 'delete' && isset($_GET['appointment_delete_id'])) {
    appointments_delete_one_appointment($object_id, 'appointments', $_GET['appointment_delete_id']);
    $step = 'start';
  }

  $object = appointments_cache_get($object_id);

  $form_state = array(
    'object_id' => $object_id,
    'object' => &$object,
  );

  $output = ctools_wizard_multistep_form($form_info, $step, $form_state);

  return $output;
}

/**
 * Helper function to delete one item from ctools cache
 */
function appointments_delete_one_item_from_cache($id, $delete_item) {
  $object = appointments_cache_get($id);
  $data = $object->$delete_item;
  if (isset($data)) {
    unset($object->$delete_item);
  }
  appointments_cache_set($id, $object);
}

/**
 * Helper function to delete one appointment
 */
function appointments_delete_one_appointment($id, $delete_item, $options = NULL) {
  $object = appointments_cache_get($id);
  if (isset($object->$delete_item)) {
    $data = $object->$delete_item;
    if (isset($data)) {
      if ($options != NULL && isset($data[$options])) {
        unset($data[$options]);
        $object->$delete_item = $data;
      }
    }
    appointments_cache_set($id, $object);
  }
}

/**
 * Handles the 'next' click on the add/edit pane form wizard.
 *
 * All we need to do is store the updated pane in the cache.
 */
function appointments_wizard_next(&$form_state) {
  appointments_cache_set($form_state['object_id'], $form_state['object']);
}


/**
 * Handles the 'finish' click on teh add/edit pane form wizard.
 *
 * All we need to do is set a flag so the return can handle adding
 * the pane.
 */
function appointments_wizard_finish(&$form_state) {
  $form_state['complete'] = TRUE;
}

/**
 * Handles the 'cancel' click on the add/edit pane form wizard.
 */
function appointments_wizard_cancel(&$form_state) {
  $form_state['cancel'] = TRUE;
}

/**
 * Generate step 1.
 */
function appointments_step1($form, &$form_state) {
  $_SESSION['old_session'] = session_id();

  $form['buttons']['next']['#access'] = FALSE;

  $form['header'] = array(
    '#markup' => '<h2>' . t("You are making an appointment for") . '</h2>',
  );

  $qmatic = new appointments_qmatic_client();
  $products = $qmatic->get_available_products();

  $options = array();
  $form_state['object']->products_durations = array();
  foreach ($products as $product) {
    $options[$product->linkId] = $product->name;
    $form_state['object']->products_durations[$product->linkId] = $product->appointLength;
  }

  if (!isset($form_state['object']->appointments)) {
    $form_state['object']->appointments = array();
  }

  // Add a default product in case a product code is passed by $_GET['product_code'].
  $default_product = (isset($_GET['product_code']) && isset($options[$_GET['product_code']])) ? $_GET['product_code'] : FALSE;
  if ($default_product && empty($form_state['object']->appointments)) {
    $form_state['object']->appointments[] = array(
      'product_link_id' => $default_product,
      'product_name' => $options[$default_product],
      'number_of_persons' => 1,
    );
  }

  $collapsed = FALSE;
  if (!empty($form_state['object']->appointments)) {
    foreach ($form_state['object']->appointments as $appointment_key => $appointment) {
      $form['appointment_info_' . $appointment_key] = array(
        '#prefix' => '<div class="added-topic">',
        '#suffix' => '</div>',
        '#submit' => 'appointment_test_submit'
      );

      if (empty($appointment['product_name'])) {
        $form_state['object']->appointments[$appointment_key]['product_name'] = $options[$appointment['product_link_id']];
      }

      $form['appointment_info_' . $appointment_key]['appointment_name'] = array(
        '#markup' => '<div class="appointment-name"><h2>' . $form_state['object']->appointments[$appointment_key]['product_name'] . '</h2></div>'
      );

      $link = l(t('Delete'), 'appointments/register/delete/', array('query' => array('appointment_delete_id' => $appointment_key)));

      $form['appointment_info_' . $appointment_key]['delete_link'] = array(
        '#markup' => $link,
        '#prefix' => '<div class="appointment-delete">',
        '#suffix' => '</div>',
      );

      $form['appointment_info_' . $appointment_key]['number_of_persons_' . $appointment_key] = array(
        '#type' => 'textfield',
        '#default_value' => (isset($appointment['number_of_persons'])) ? $appointment['number_of_persons'] : 1,
        '#size' => 4,
        '#maxlength' => 4,
        '#prefix' => '<div class="appointment-persons"><label>' . t('Number of persons') . '</label>',
        '#suffix' => '</div>',
      );

      $form['appointment_info_' . $appointment_key]['appointment_id_' . $appointment_key] = array(
        '#type' => 'value',
        '#value' => $appointment['product_link_id'],
      );


    }
    $collapsed = TRUE;
    $form['buttons']['next']['#access'] = TRUE;
  }

  if (isset($form_state['object']->step1_button['clicked_button']) && $form_state['object']->step1_button['clicked_button'] == 'add_new_one') {
    appointments_delete_one_item_from_cache('appointments', 'step1_button');
    $form_state['input'] = array();
    $form_state['post'] = array();
  }

  $appointment_added = isset($form_state['object']->appointments) && !empty($form_state['object']->appointments) ? TRUE : FALSE;

  $form['new_topic'] = array(
    '#type' => 'fieldset',
    '#title' => $appointment_added ? t('Add New Topic') : '',
    '#collapsible' => $collapsed,
    '#collapsed' => $collapsed,
    '#attributes' => array('class' => array('new-topic-fieldset')),
  );

  $form['new_topic']['subject'] = array(
    '#type' => 'select',
    '#title' => t('Subject'),
    '#options' => $options,
    '#empty_option' => t('-- Choose a product --'),
    '#required' => $appointment_added ? FALSE : TRUE,
  );

  $form['new_topic']['number_of_persons'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of persons'),
    '#size' => 4,
    '#default_value' => 1,
    '#maxlength' => 4,
    '#required' => $appointment_added ? FALSE : TRUE,
  );

  $form['new_topic']['add_new_one'] = array(
    '#type' => 'submit',
    '#value' => t('Add subject'),
    '#name' => 'add_new_one',
    '#submit' => array('appointments_add_new_one_submit'),
    '#validate' => array('appointments_add_new_one_validate'),
    '#attributes' => array('class' => array('add-new-one')),
  );

  return $form;
}

/**
 * Validate function for the Add_new_one button.
 */
function appointments_add_new_one_validate($form, &$form_state) {
  if (isset($form_state['object']->appointments) && !empty($form_state['object']->appointments)) {
    foreach ($form_state['object']->appointments as $appointment) {
      if ($appointment['product_link_id'] == $form_state['values']['subject']) {
        form_set_error('subject', t('You already choose this product.'));
      }
    }
  }

  if ($form_state['values']['subject'] == 'NULL') {
    form_set_error('subject', t('Please, chose one of the product.'));
  }

  if ($form_state['values']['number_of_persons'] <= 0 || $form_state['values']['number_of_persons'] > 5 || preg_match('/(d+)/', $form_state['values']['number_of_persons'])) {
    form_set_error('number_of_persons', t('Please, enter a valid number of persons. The number of persons should not be more then 5 persons.'));
  }

  $count_appointments = isset($form_state['object']->appointments) ? count($form_state['object']->appointments) : 0;
  for ($i = 0; $i < $count_appointments; $i++) {
    if (isset($form_state['values']['number_of_persons_' . $i])) {
      if ($form_state['values']['number_of_persons_' . $i] <= 0 || $form_state['values']['number_of_persons_' . $i] > 5 || preg_match('/(d+)/', $form_state['values']['number_of_persons_' . $i])) {
        form_set_error('number_of_persons_' . $i, t('Please, enter a valid number of persons. The number of persons should not be more then 5 persons.'));
      }
    }
  }
}

/**
 * Submit function for the Add_new_one button.
 */
function appointments_add_new_one_submit($form, &$form_state) {
  $form_state['next'] = 'start';
  $form_state['object']->appointments[] = array(
    'product_link_id' => $form_state['values']['subject'],
    'product_name' => $form['new_topic']['subject']['#options'][$form_state['values']['subject']],
    'number_of_persons' => $form_state['values']['number_of_persons'],
  );
  $form_state['object']->step1_button['clicked_button'] = $form_state['clicked_button']['#name'];

  if (isset($form_state['object']->appointments) && !empty($form_state['object']->appointments)) {
    $form_state['object'] = _appointment_check_number_of_persons_field($form_state['values'], $form_state['object']);
  }
  appointments_cache_set('appointments', $form_state['object']);

  drupal_goto('appointments/register/start');
}

/*
 * Validate function step 1.
 */
function appointments_step1_validate($form, &$form_state) {
  // Add the product even if the add_new_one form isn't submitted.
  if ($form_state['values']['subject'] != ''){
    appointments_add_new_one_validate($form, $form_state);
  }

  $count_appointments = count($form_state['object']->appointments);
  for ($i = 0; $i < $count_appointments; $i++) {
    if (isset($form_state['values']['number_of_persons_' . $i])) {
      if ($form_state['values']['number_of_persons_' . $i] <= 0 || $form_state['values']['number_of_persons_' . $i] > 5 || preg_match('/(d+)/', $form_state['values']['number_of_persons_' . $i])) {
        form_set_error('number_of_persons_' . $i, t('Please, enter a valid number of persons. The number of persons should not be more then 5 persons.'));
      }
    }
  }
}

/**
 * Submit function for the Add_new_one button.
 */
function appointments_step1_submit($form, &$form_state) {
  // Add the product even if the add_new_one form isn't submitted.
  if ($form_state['values']['subject'] != ''){
    $form_state['object']->appointments[] = array(
      'product_link_id' => $form_state['values']['subject'],
      'product_name' => $form['new_topic']['subject']['#options'][$form_state['values']['subject']],
      'number_of_persons' => $form_state['values']['number_of_persons'],
    );
  }

  if (isset($form_state['object']->appointments) && !empty($form_state['object']->appointments)) {
    $form_state['object'] = _appointment_check_number_of_persons_field($form_state['values'], $form_state['object']);
  }

  appointments_cache_set('appointments', $form_state['object']);
}

/**
 * Helper function that check number_of_persons field
 */
function _appointment_check_number_of_persons_field($values, $object) {
  $count_appointments = count($object->appointments);

  for ($i = 0; $i < $count_appointments; $i++) {
    if (isset($values['number_of_persons_' . $i]) && !empty($values['number_of_persons_' . $i])) {
      foreach ($object->appointments as $key => $appointment) {
        if ($appointment['product_link_id'] == $values['appointment_id_' . $i]) {
          if ($appointment['number_of_persons'] != $values['number_of_persons_' . $i]) {
            $object->appointments[$key]['number_of_persons'] = $values['number_of_persons_' . $i];
          }
        }
      }
    }
  }
  return $object;
}

/**
 * Generate Step 2.
 */
function appointments_step2($form, &$form_state) {
  $_SESSION['old_session'] = session_id();

  // Redirect to step 1 in case no appointment data is available at this point.
  if(!isset($form_state['object']->appointments)) {
    drupal_goto('appointments/register/start');
  }

  _appointments_fix_previous_button($form);

  $form['header'] = array(
    '#markup' => '<h2>' . t("Choose a date and time") . '</h2>',
  );

  $rows = array();
  $persons = array();
  $product_link_ids = array();
  if (isset($form_state['object'])) {
    foreach ($form_state['object']->appointments as $form_field_value) {
      $total_product_duration = 0;
      foreach ($form_state['object']->products_durations as $product_id => $duration) {
        if ($product_id == $form_field_value['product_link_id']) {
          $total_product_duration += ($duration * $form_field_value['number_of_persons']);
        }
      }

      $rows[] = array(
        'product_name' => $form_field_value['product_name'],
        'duration' => $total_product_duration,
      );

      if (isset($form_field_value['product_link_id']) && !empty($form_field_value['product_link_id'])) {
        $product_link_ids[] = $form_field_value['product_link_id'];
      }

      $persons[$form_field_value['product_link_id']] = $form_field_value['number_of_persons'];
    }
  }

  $output_info = _appointments_get_step2_header_text($rows);

  $form['info_table'] = array(
    '#title' => t('Information table'),
    '#markup' => $output_info,
  );

  $form['data_table'] = array(
    '#theme' => 'appointments_form_date_table',
  );

  $dates_all = _appointments_get_all_right_dates_times($product_link_ids, $form_state['object']->products_durations, $persons);
  $form_state['object']->dates_all_magic = $dates_all;
  $dates = array();
  foreach ($dates_all as $unic_id => $date_array) {
    if ($unic_id == 0 || !($date_array[0]['date'] == $dates_all[$unic_id - 1][0]['date'])) {
      $dates[$date_array[0]['date']] = array();
    }
    $dates[$date_array[0]['date']][$unic_id] = $date_array[0]['time'];
  }

  if (APPOINTMENTS_DEBUG_MODE) {
    watchdog('appointments', 'dates ->' . var_export($dates, TRUE));
  }

  $full_date = array();
  $i = 0;
  foreach ($dates as $date => $time_array) {
    $unix_date = strtotime($date);
    $morning_time = array();
    $midday_time = array();
    $evening_time = array();
    $full_time = array();

    $options = array('NULL' => t('-- Select time --'));

    foreach ($time_array as $unic_id_of_date => $time) {
      $unix_time = strtotime($time);
      $midday = gmmktime(12, 00, 0, 1, 1, 1970);
      $evening = gmmktime(17, 00, 0, 1, 1, 1970);


      if ($unix_time < $midday) {
        $morning_time[$unic_id_of_date] = appointments_format_date($unix_time, 'time');
      }
      elseif ($unix_time >= $midday && $unix_time < $evening) {
        $midday_time[$unic_id_of_date] = appointments_format_date($unix_time, 'time');
      }
      elseif ($unix_time >= $evening) {
        $evening_time[$unic_id_of_date] = appointments_format_date($unix_time, 'time');
      }

      if (!empty($morning_time)) {
        $full_time['morning'] = TRUE;
        $options[t('Morning')] = $morning_time;
      }
      if (!empty($midday_time)) {
        $full_time['midday'] = TRUE;
        $options[t('Midday')] = $midday_time;
      }
      if (!empty($evening_time)) {
        $full_time['evening'] = TRUE;
        $options[t('Evening')] = $evening_time;
      }
    }

    $form['data_table']['time_' . $i] = array(
      '#title' => t('Time'),
      '#title_display' => 'invisible',
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => isset($form_state['object']->forma_date_info['form_name']) && $form_state['object']->forma_date_info['form_name'] == $i ? $form_state['object']->forma_date_info['form_value'] : NULL,
    );

    $i++;

    $full_date[] = array(
      'date' => $unix_date,
      'date_time' => $full_time,
    );
  }

  $form['data_table']['appointments_full_date'] = array(
    '#type' => 'value',
    '#value' => $full_date,
  );

  return $form;

}

/**
 * Validate function for the appointments_step2 form.
 */
function appointments_step2_validate($form, &$form_state) {
  $times = count($form_state['values']['appointments_full_date']);
  unset($form_state['values']['appointments_full_date']);
  $date = array();

  for ($i = 0; $i < $times; $i++) {
    if ($form_state['values']['time_' . $i] == 'NULL') {
      unset($form_state['values']['time_' . $i]);
    }
    else {
      $date[] = $i;
      $form_state['values']['date'] = $form_state['values']['time_' . $i];
      $form_state['values']['date_form_name'] = $i;
      $form_state['values']['date_form_value'] = $form_state['values']['time_' . $i];
    }
  }

  if (empty($date)) {
    form_set_error('', t('You should select time for you appointment.'));
  }
  elseif (count($date) > 1) {
    form_set_error('', t('You should choose time only for one day.'));
  }
}

/**
 * Submit function for the appointments_step2 form.
 */
function appointments_step2_submit($form, &$form_state) {
  $form_state['object']->date_data_last = $form_state['object']->dates_all_magic[$form_state['values']['date']];

  if (APPOINTMENTS_DEBUG_MODE) {
    watchdog('appointments', 'date_data_last -> ' . var_export($form_state['object']->date_data_last, TRUE));
  }

  $form_state['object']->forma_date_info = array(
    'form_name' => $form_state['values']['date_form_name'],
    'form_value' => $form_state['values']['date_form_value'],
  );
  unset($form_state['object']->dates_all_magic);
}

/**
 * Form with tabs for the third step.
 */
function appointments_step3($form, &$form_state) {
  global $user;

  if (isset($_SESSION['old_session']) && $_SESSION['old_session'] != session_id()) {
    $form_state['object'] = appointments_cache_get('appointments', $_SESSION['old_session']);
  }

  // Redirect to step 1 in case no appointment data is available at this point.
  if(!isset($form_state['object']->appointments)) {
    drupal_goto('appointments/register/start');
  }

  _appointments_fix_previous_button($form);

  $digid_user = _dvg_digid_logged_in($user);

  $email = '';
  if (isset($form_state['object']->user_information['email'])) {
    $email = $form_state['object']->user_information['email'];
  }

  $form['header'] = array(
    '#markup' => '<h2>' . t('Your details') . '</h2><p>' . t("You can also login by DigiD. You don't have to fill in your name and address information anymore.") . '</p>',
  );

  drupal_add_js(drupal_get_path('module', 'appointments') . '/js/appointments.js');

  $tab_list = array(
    l(t('Enter your information'), current_path(), array('fragment' => 'section-1', 'attributes' => array('id' => 'section-1-tab','class' => array(!$digid_user ? 'selected' : '')))),
    l(t('Log in with DigiD'), current_path(), array('fragment' => 'section-2', 'attributes' => array('id' => 'section-2-tab', 'class' => array($digid_user ? 'selected' : '')))),
  );

  $form['tabs'] = array(
    '#prefix' => '<div class="section-list">',
    '#markup' => theme('item_list', array('items' => $tab_list)),
    '#suffix' => '</div>',
  );

  $form['tab_1'] = array(
    '#type' => 'container',
    '#attributes' => array('id' => 'section-1-content', 'class' => array('section')),
  );

  $form['tab_1']['header'] = array(
    '#markup' => '<h2>' . t('Enter your information'). '</h2>',
  );

  $form['tab_1']['first_name'] = array(
    '#type' => 'textfield',
    '#type' => 'textfield',
    '#title' => t('First name'),
    '#required' => !$digid_user,
    '#default_value' => isset($form_state['object']->user_information['first_name']) ? $form_state['object']->user_information['first_name'] : '',
  );

  $form['tab_1']['surname'] = array(
    '#type' => 'textfield',
    '#title' => t('Surname'),
    '#required' => !$digid_user,
    '#default_value' => isset($form_state['object']->user_information['second_name']) ? $form_state['object']->user_information['second_name'] : '',
  );

  $form['tab_1']['date_of_birth'] = array(
    '#type' => 'date',
    '#title' => t('Date of birth'),
    '#date_format' => 'd-m-Y',
    '#start_year' => -100,
    '#end_year' => +0,
    '#required' => !$digid_user,
    '#default_value' => isset($form_state['object']->user_information['born']) ? $form_state['object']->user_information['born'] : array('year' => 0, 'month' => 0, 'day' => 0),
    '#process' => array('form_process_date', '_appointments_process_date_of_birth'),
    '#element_validate' => array(),
  );

  $form['tab_1']['email'] = array(
    '#type' => 'textfield',
    '#title' => t('E-mail'),
    '#description' => t('Fill in your email address, please You will receive confirmation email with your appointment number. With this number you can cancel your appointment yet.'),
    '#required' => !$digid_user,
    '#default_value' => $email,
  );

  $form['tab_1']['telephone_number'] = array(
    '#type' => 'textfield',
    '#title' => t('Telephone number (optional)'),
    '#description' => t('Please fill your phone number. We can quickly contact you in case of problems or confusions.'),
    '#required' => FALSE,
    '#default_value' => isset($form_state['object']->user_information['phone']) ? $form_state['object']->user_information['phone'] : '',
  );

  $form['tab_2'] = array(
    '#type' => 'container',
    '#attributes' => array('id' => 'section-2-content', 'class' => array('section')),
  );

  $form['tab_2']['header'] = array(
    '#markup' => '<h2>' . t('Log in with DigiD'). '</h2>',
  );

  $form['tab_2']['digid_email'] = array(
    '#type' => 'textfield',
    '#title' => t('E-mail'),
    '#description' => t('Fill in your email address, please You will receive confirmation email with your appointment number. With this number you can cancel your appointment yet.'),
    '#required' => $digid_user,
    '#access' => $digid_user,
    '#default_value' => $email,
  );

  if (!$digid_user) {
    $destination_alias = drupal_get_path_alias('appointments/register/third', NULL);

    $digid_logo = '';
    $fid = variable_get('dvg_digid_logo', FALSE);
    if ($fid) {
      $file = file_load($fid);
      $image = theme_image_style(array(
        'style_name' => 'digid_logo',
        'path' => $file->uri,
        'width' => NULL,
        'height' => NULL,
        'alt' => t('DigiD logo'),
        'title' => t('DigiD logo'),
      ));

      $digid_logo = '<span class="digid-logo">' . $image . '</span>';
    }

    $form['tab_2']['digid_link'] = array(
      '#markup' => l(t('Log in with DigiD'), 'user/digid', array('query' => array('destination' => $destination_alias), 'attributes' => array('class' => array('btn')))),
      '#prefix' => '<div class="digid">',
      '#suffix' => $digid_logo . '</div>',
    );
  }

  $form['tab_1']['buttons'] =
  $form['tab_2']['buttons'] = $form['buttons'];
  unset($form['buttons']);

  if (!$digid_user) {
    $form['tab_2']['buttons']['next']['#access'] = FALSE;
  }

  return $form;
}

/**
 * Validate function for the appointments_step3 form.
 */
function appointments_step3_validate($form, &$form_state) {
  global $user;
  $digid_user = _dvg_digid_logged_in($user);

  if ($digid_user) {
    // Validate email.
    if ($form['tab_2']['digid_email']['#required'] && !valid_email_address($form_state['values']['digid_email'])) {
      form_set_error('digid_email', t('The e-mail address you specified is not valid.'));
    }
  }
  else {
    // Validate a date of birth.
    $date_of_birth = isset($form_state['values']['date_of_birth']) ? $form_state['values']['date_of_birth'] : array();
    if ($form['tab_1']['date_of_birth']['#required'] && !checkdate($date_of_birth['month'], $date_of_birth['day'], $date_of_birth['year'])) {
      form_set_error('date_of_birth', t('The date of birth you specified is not valid.'));
    }

    // Validate email.
    if ($form['tab_1']['email']['#required'] && !valid_email_address($form_state['values']['email'])) {
      form_set_error('email',  t('The e-mail address you specified is not valid.'));
    }

    // lenient checking, as long as don't have invalid phone number character
    $regex = '/^
      [\s.()-]*     # optional separator
      (?:           # }
        \d          # } 4-15 digits number
        [\s.()-]*   # } each followed by optional separator
      ){' . 4 . ',' . 15 . '}       # }
      $/x';

    if ($form_state['values']['telephone_number'] != '' && !preg_match($regex, $form_state['values']['telephone_number'])) {
      form_set_error('telephone_number', t('Phone number must be %min_length-%max_length digits only.', array('%min_length' => 4, '%max_length' => 15)));
    }
  }
}

/**
 * Submit function for the appointments_step3 form.
 */
function appointments_step3_submit($form, &$form_state) {
  $user = array();

  $bsn = dvg_digid_get_bsn();
  if ($bsn) {
    $user['first_name'] = '';
    $user['second_name'] = '';
    $user['born'] = '';
    $user['phone'] = '';
    $user['email'] = !empty($form_state['values']['digid_email']) ? $form_state['values']['digid_email'] : '';
    $user['piNo'] = md5($form_state['values']['digid_email']);
    $user['bsn'] = $bsn;
  }
  else {
    $user['first_name'] = $form_state['values']['first_name'];
    $user['second_name'] = $form_state['values']['surname'];
    $user['born'] = $form_state['values']['date_of_birth'];
    $user['phone'] = $form_state['values']['telephone_number'];
    $user['email'] = (!empty($form_state['values']['email'])) ? $form_state['values']['email'] : $form['tab_1']['email']['#default_value'];
    $user['piNo'] = md5($form_state['values']['email']);
  }

  $form_state['object']->user_information = $user;
}

/**
 * Generate step 4
 */
function appointments_step4($form, &$form_state) {
  // Redirect to step 1 in case no appointment data is available at this point.
  if(!isset($form_state['object']->appointments)) {
    drupal_goto('appointments/register/start');
  }

  _appointments_fix_previous_button($form);

  $form['header'] = array(
    '#markup' => '<h2>' . t('Check and confirm your appointment') . '</h2>',
  );

  foreach ($form_state['object']->appointments as $appointment) {
    $total_duration = 0;
    $start_time = 0;
    foreach ($form_state['object']->date_data_last as $app_data) {
      if ($app_data['product_id'] == $appointment['product_link_id']) {
        $total_duration += $app_data['duration'];

        if (!$start_time) {
          $start_time = strtotime($app_data['time']);
        }
      }
    }
    $start_date = strtotime($form_state['object']->date_data_last[0]['date']);
    $end_time = $start_time + ($total_duration * 60);

    $first_info_block = array();
    $first_info_block[] = array(
      'label' => t('Subject'),
      'value' => $appointment['product_name'],
    );
    $first_info_block[] = array(
      'label' => t('Number of persons'),
      'value' => $appointment['number_of_persons'],
    );
    $first_info_block[] = array(
      'label' => t('Date'),
      'value' => appointments_format_date($start_date, 'dvg_short_date'),
    );
    $first_info_block[] = array(
      'label' => t('Time'),
      'value' => t('@start till @end', array('@start' => appointments_format_date($start_time, 'time'), '@end' => appointments_format_date($end_time, 'time'))),
    );

    $table_first = theme('appointments_info_table', $first_info_block);

    $form['info_table_first' . $appointment['product_link_id']] = array(
      '#markup' => $table_first,
    );
  }

  $second_info_block = array();
  if (isset($form_state['object']->user_information)) {
    if ($form_state['object']->user_information['first_name']) {
      $second_info_block[] = array(
        'label' => t('First name'),
        'value' => $form_state['object']->user_information['first_name'],
      );
    }
    if ($form_state['object']->user_information['second_name']) {
      $second_info_block[] = array(
        'label' => t('Second name'),
        'value' => $form_state['object']->user_information['second_name'],
      );
    }

    if (!empty($form_state['object']->user_information['born'])) {
      $date_born = $form_state['object']->user_information['born']['year'] . '-' . $form_state['object']->user_information['born']['month'] . '-' . $form_state['object']->user_information['born']['day'];
      if ($form_state['object']->user_information['born']) {
        $second_info_block[] = array(
          'label' => t('Birthday'),
          'value' => format_date(strtotime($date_born), 'dvg_short_date'),
        );
      }
    }
    if ($form_state['object']->user_information['email']) {
      $second_info_block[] = array(
        'label' => t('E-mail'),
        'value' => $form_state['object']->user_information['email'],
      );
    }
    if ($form_state['object']->user_information['phone']) {
      $second_info_block[] = array(
        'label' => t('Phone'),
        'value' => $form_state['object']->user_information['phone'],
      );
    }
    $table_second = theme('appointments_info_table', $second_info_block);

    $form['info_table_second'] = array(
      '#markup' => $table_second,
      '#prefix' => '<h2>' . t('Your data') . '</h2>',
    );
  }

  return $form;
}

/**
 * Submit function for the appointments_step4 form.
 */
function appointments_step4_submit($form, &$form_state) {
  $qmatic = new appointments_qmatic_client();
  $form_state['object']->book_data = array();

  foreach ($form_state['object']->date_data_last as $app_data) {
    $born = FALSE;
    if (!empty($form_state['object']->user_information['born'])) {
      $born = $form_state['object']->user_information['born']['year'] . '-' . $form_state['object']->user_information['born']['month'] . '-' . $form_state['object']->user_information['born']['day'];
    }

    $name_values = array();
    if (isset($form_state['object']->user_information['first_name']) && $form_state['object']->user_information['first_name'] != '') {
      $name_values[] = $form_state['object']->user_information['first_name'];
    }
    if (isset($form_state['object']->user_information['second_name']) && $form_state['object']->user_information['second_name'] != '') {
      $name_values[] = $form_state['object']->user_information['second_name'];
    }
    $full_name = implode(' ', $name_values);

    $bsn = isset($form_state['object']->user_information['bsn']) ? $form_state['object']->user_information['bsn'] : FALSE;

    $book_info = $qmatic->bookAppointment(
      array(
        'productLinkID' => $app_data['product_id'],
        'appDate' => $app_data['date'],
        'appTime' => $app_data['time'],
        'name' => $bsn ? ('BSN: ' . $bsn) : $full_name,
        'phone' => $form_state['object']->user_information['phone'],
        'email' => $form_state['object']->user_information['email'],
        'piNo' => $form_state['object']->user_information['piNo'],
        'born' => $born ? format_date(strtotime($born), 'custom', 'd-m-Y') : '',
      )
    );

    $form_state['object']->book_data[] = array(
      'product_id' => $app_data['product_id'],
      'book_status' => $book_info->appointmentStatus,
      'book_id' => $book_info->appointmentId,
    );

    $mail_array = array(
      '%id' => trim($book_info->appointmentId),
      '%subject' => '',
      '%estimated_duration' => $app_data['duration'],
      '%date' => appointments_format_date(strtotime($app_data['date']), 'dvg_short_date'),
      '%start_time' => appointments_format_date(strtotime($app_data['time']), 'time'),
      '%end_time' => appointments_format_date(strtotime($app_data['time']) + ($app_data['duration'] * 60), 'time'),
      '%first_name' => isset($form_state['object']->user_information['first_name']) ? $form_state['object']->user_information['first_name'] : '',
      '%surname' => isset($form_state['object']->user_information['second_name']) ? $form_state['object']->user_information['second_name'] : '',
      '%date_of_birth' => $born ? format_date(strtotime($born), 'custom', 'd-m-Y') : '',
      '%email' => $form_state['object']->user_information['email'],
      '%telephone_number' => $form_state['object']->user_information['phone'],
      '%bsn' => $bsn ? $bsn : '',
      '%cancel_appointment_link' => url('annuleren-afspraak', array('absolute' => TRUE)),
    );

    // Task confirmation text.
    $mail_array['%field_confirm_appointment'] = '';
    $task = _appointments_get_task_by_product_code($app_data['product_id']);
    if ($task) {
      $field_confirm_appointment = field_get_items('node', $task, 'field_confirm_appointment', $task->language);
      if (!empty($field_confirm_appointment)) {
        $mail_array['%task_confirm_text'] = trim(drupal_html_to_text($field_confirm_appointment[0]['value']));
      }
    }
    if (!isset($mail_array['%task_confirm_text'])) {
      $mail_array['%task_confirm_text'] = '';
    }

    foreach ($form_state['object']->appointments as $appointment) {
      if ($app_data['product_id'] == $appointment['product_link_id']) {
        $mail_array['%subject'] = $appointment['product_name'];
      }
    }
    appointments_confirmation_mail_send($form_state['object']->user_information['email'], $mail_array);

    $second_email = variable_get('appointments_second_email', '');
    if (!empty($second_email)) {
      appointments_confirmation_mail_send($second_email, $mail_array);
    }
  }
}

/**
 * Generate the last step Step 5
 */
function appointments_step5($form, &$form_state) {
  // Redirect to step 1 in case no appointment data is available at this point.
  if(!isset($form_state['object']->appointments)) {
    drupal_goto('appointments/register/start');
  }

  if (isset($form['buttons']['previous']) && isset($form['buttons']['return'])) {
    unset($form['buttons']);
  }

  $form['header'] = array(
    '#markup' => '<h2>' . t('Your appointment has been created') . '</h2>',
  );

  if (isset($form_state['object']->appointments)) {
    foreach ($form_state['object']->appointments as $appointment) {
      $info_table_html = '';
      foreach ($form_state['object']->date_data_last as $i => $app_data) {
        if ($appointment['product_link_id'] == $app_data['product_id']) {
          $book_info = $form_state['object']->book_data[$i];

          // Display an error message in case the appointment booking failed.
          if ($book_info['book_status'] != '1') {
            switch ($book_info['book_status']) {
              case '1':
                $status_text = t('Appointment booked');
                break;
              case '2':
                $status_text = t('Invalid product selection');
                break;
              case '3':
                $status_text = t('Invalid date selection');
                break;
              case '4':
                $status_text = t('Invalid time selection');
                break;
              case '5':
              default:
                $status_text = t('An unknown error occurred');
                break;
            }

            drupal_set_message($status_text, 'error');
          }

          $info_block = array();

          $date = strtotime($form_state['object']->date_data_last[0]['date']);
          $start_time = strtotime($app_data['time']);
          $end_time = $start_time + ($app_data['duration'] * 60);

          $info_block[] = array(
            'label' => t('Subject'),
            'value' => $appointment['product_name'],
          );

          $info_block[] = array(
            'label' => t('Number of persons'),
            'value' => '1',
          );

          $info_block[] = array(
            'label' => t('Date'),
            'value' => appointments_format_date($date, 'dvg_short_date'),
          );

          $info_block[] = array(
            'label' => t('Time'),
            'value' => t('@start till @end', array('@start' => appointments_format_date($start_time, 'time'), '@end' => appointments_format_date($end_time, 'time'))),
          );

          $info_block[] = array(
            'label' => t('Appointment id'),
            'value' => ($book_info['book_id'] == 0) ? '' : $book_info['book_id'],
          );

          $info_table_html .= theme('appointments_info_table', $info_block);
        }
      }

      $info_table = array(
        '#markup' => $info_table_html,
        '#prefix' => '<div class="appointments-info-table-container">',
        '#suffix' => '</div>',
      );

      // Display the Task appointment confirmation text per selected product.
      $task_confirmation_text = array();
      $task = _appointments_get_task_by_product_code($appointment['product_link_id']);
      if ($task) {
        $field_confirm_appointment = field_get_items('node', $task, 'field_confirm_appointment', $task->language);
        if (!empty($field_confirm_appointment)) {
          $task_confirmation_text = array(
            '#markup' => check_markup($field_confirm_appointment[0]['value'], $field_confirm_appointment[0]['format']),
            '#prefix' => '<div class="appointments-confirmation-text">',
            '#suffix' => '</div>',
          );
        }
      }

      $form['information_block'][] = array(
        '#markup' => render($info_table) .  render($task_confirmation_text),
      );
    }
  }

  $cancel_link = l(t('change or cancel'), 'appointments/delete/');
  $form['footer_description'] = array(
    '#markup' => '<h2>' . t('Change or cancel') . '</h2><p>' . t('You will also receive this information by email. Save this email carefully. With your appointment ID and email address you can !cancel_link.', array('!cancel_link' => $cancel_link)) . '</p>',
  );

  appointments_cache_clear('appointments');

  return $form;
}

/**
 * Theming function for the info tables.
 */
function appointments_info_table_theme_list($row_data) {
  $html = '';
  if (is_array($row_data)) {
    $html .= "<ul>";
    foreach ($row_data as $key => $data) {
      if (is_array($data)) {
        $data = appointments_info_table_theme_list($data);
      }
      $html .= "<li>" . $data . "</li>";
    }
    $html .= "</ul>";
  }
  else {
    $html .= $row_data;
  }

  return $html;
}

/**
 * Theming function for the date table on step 2.
 */
function theme_appointments_form_date_table($vars) {
  $output = '';
  $data_table = $vars['data_table'];
  $appointments_full_date = $data_table['appointments_full_date'];

  $day_parts = array(
    'morning' => t('Morning'),
    'midday' => t('Midday'),
    'evening' => t('Evening'),
  );

  if (isset($appointments_full_date['#value']) && !empty($appointments_full_date['#value'])) {
    $table_data = array(
      'header' => array(
        array('data' => t('Date')),
        array('data' => $day_parts['morning']),
        array('data' => $day_parts['midday']),
        array('data' => $day_parts['evening']),
        array('data' => '<span class="element-invisible">' . t('Time') . '</span>'),
      ),
      'rows' => array(),
      'attributes' => array(
        'class' => array('appointments-date-selection-table'),
      ),
      'sticky' => FALSE,
    );

    $i = 0;
    foreach ($appointments_full_date['#value'] as $date) {
      $table_row = array();

      $table_row[] = array('data' => appointments_format_date($date['date'], 'dvg_short_date'), 'class' => 'date');

      foreach ($day_parts as $day_part_key => $day_part_label) {
        $day_part_available = isset($date['date_time'][$day_part_key]) ? TRUE : FALSE;
        $image_class = $day_part_available ? 'available' : 'unavailable';
        $day_part_prefix = '<span class="day-part-label">' . $day_part_label . '</span>';
        $day_part_status = array(
          TRUE => array(
            'icon' => 'icon-tick.png',
            'alt' => t('available'),
          ),
          FALSE => array(
            'icon' => 'icon-cross.png',
            'alt' => t('unavailable'),
          ),
        );
        $image = array(
          'path' => drupal_get_path('module', 'appointments') . '/images/' . $day_part_status[$day_part_available]['icon'],
          'alt' => $day_part_status[$day_part_available]['alt'],
        );
        $day_part_image = '<span class="day-part-image ' . $image_class . '">' . theme('image', $image) . '</span>';
        $day_part_content = '<div class="day-part">' . $day_part_prefix . ' ' . $day_part_image . '</div>';
        $table_row[] = array(
          'data' => $day_part_content,
          'class' => array('time-appointments', $day_part_key)
        );
      }

      $table_row[] = array('data' => render($data_table['time_' . $i]), 'class' => 'time-selector-appointments');

      $table_data['rows'][] = $table_row;

      $i++;
    }

    $output .= theme('table', $table_data);
  }
  else {
    $markup = array(
      '#markup' => t('No suitable date and time for this request. Please register appointments separately.'),
      '#prefix' => '<div class="time_error_step2">',
      '#suffix' => '</div>',
      '#weight' => 4,
    );
    $output .= render($markup);
  }

  return $output;
}

/**
 * Helper function which get all right dates for appointments.
 */
function _appointments_get_all_right_dates_times($product_link_ids, $durations, $persons) {
  $qmatic = new appointments_qmatic_client();
  $dates_all = array();
  foreach ($product_link_ids as $product_link_id) {
    foreach ($qmatic->get_available_appointment_days($product_link_id) as $date) {
      $time = $qmatic->get_available_appointment_times($product_link_id, $date);
      if (!isset($dates_all[$product_link_id])) {
        $dates_all[$product_link_id] = array();
        $dates_all[$product_link_id]['dates'] = array();
        $dates_all[$product_link_id]['product_id'] = $product_link_id;
        $dates_all[$product_link_id]['duration'] = $durations[$product_link_id];
      }
      if (!isset($dates_all[$product_link_id]['dates'][$date])) {
        $dates_all[$product_link_id]['dates'][$date] = is_array($time) ? $time : array($time);
      }
    }
  }

  if (APPOINTMENTS_DEBUG_MODE) {
    watchdog('appointments', 'dates_all -> ' . var_export($dates_all, TRUE));
  }

  $available_dates = _appointments_check_dates($dates_all);
  // _appointments_magic checks the dates.
  $right_dates_time = _appointments_magic($dates_all, $available_dates, $persons);

  if (APPOINTMENTS_DEBUG_MODE) {
    watchdog('appointments', 'right_dates_time -> ' . var_export($right_dates_time, TRUE));
  }

  return $right_dates_time;
}

/**
 * Helper function which check the same date for appointments.
 */
function _appointments_check_dates($dates_all) {
  $right_dates = array();

  $first_dates = array_shift($dates_all);
  foreach ($first_dates['dates'] as $date => $time) {
    $right_dates[] = $date;
  }
  foreach ($dates_all as $product) {
    $right_dates_temp = $right_dates;
    $right_dates = array();
    foreach ($product['dates'] as $date => $time) {
      if (in_array($date, $right_dates_temp)) {
        $right_dates[] = $date;
      }
    }
  }
  return $right_dates;
}

/**
 * Magic function that check all correct time for appointments.
 */
function _appointments_magic($dates_all, $available_dates = array(), $persons = array()) {
  $good_times = array();
  $dates = array_shift($dates_all);
  foreach ($available_dates as $date) {
    foreach ($dates['dates'][$date] as $time) {
      $person_is_ok = array();
      $appointments = array();
      for ($i = 0; $i < $persons[$dates['product_id']]; $i++) {
        // Why was ($i+1) ? Deleted because 9;00 was not visible
        $search_time = strtotime($time) + $dates['duration'] * 60 * ($i);
        foreach ($dates['dates'][$date] as $_time) {
          if ($search_time == strtotime($_time)) {
            $person_is_ok[$i] = TRUE;
            $appointments[] = array(
              'product_id' => $dates['product_id'],
              'duration' => $dates['duration'],
              'date' => $date,
              'time' => $_time,
            );
          }
        }
      }

      // Is this oke? Then check for person 1
      $is_ok = TRUE;
      for ($p = 0; $p < $persons[$dates['product_id']]; $p++) {
        if (!isset($person_is_ok[$p])) {
          $is_ok = FALSE;
        }
      }

      if ($is_ok) {
        $good_times[] = $appointments;
      }
    }
  }

  if (APPOINTMENTS_DEBUG_MODE) {
    watchdog('appointments', 'good_times -> ' . var_export($good_times, TRUE));
  }

  foreach ($dates_all as $dates) {
    $good_times_temp = $good_times;
    $good_times = array();
    foreach ($available_dates as $date) {
      foreach ($dates['dates'][$date] as $time) {
        foreach ($good_times_temp as $appointments) {
          $last_appointment = end($appointments);

          if ($date == $last_appointment['date'] && (strtotime($time) == strtotime($last_appointment['time']) + $last_appointment['duration'] * 60)) {
            $person_is_ok = array();
            for ($i = 1; $i <= $persons[$dates['product_id']]; $i++) {
              // This part should check the next moment that is available. The for-loop should start.
              $search_time = strtotime($last_appointment['time']) + $last_appointment['duration'] * 60 * ($i);
              foreach ($dates['dates'][$date] as $_time) {
                if ($search_time == strtotime($_time)) {
                  $person_is_ok[$i] = TRUE;
                  $appointments[] = array(
                    'product_id' => $dates['product_id'],
                    'duration' => $dates['duration'],
                    'date' => $date,
                    'time' => $_time,
                  );
                }
              }
            }

            // Is this OK? Then start with 1 because that should be added ($i) to the time in the above loop to find the next moment.
            $is_ok = TRUE;
            for ($o = 1; $o <= $persons[$dates['product_id']]; $o++) {
              if (!isset($person_is_ok[$o])) {
                $is_ok = FALSE;
              }
            }

            if ($is_ok) {
              $good_times[] = $appointments;
            }
          }
        }
      }
    }
  }

  // Returns al goed times that need to be choosable.
  return $good_times;
}

/**
 * Helper function for displaying the step 2 header text.
 */
function _appointments_get_step2_header_text($selected_products) {
  $product_text = '';
  $total_duration = 0;
  $amount = count($selected_products);
  foreach ($selected_products as $i => $data) {
    $separator = '';
    if ($i > 0 && $amount > 1) {
      $separator = ($i + 1) < $amount ? ', ' : ' ' . t('and') . ' ';
    }

    $product_text .= $separator . '<strong>' . check_plain($selected_products[$i]['product_name']) . '</strong>';
    $total_duration += $selected_products[$i]['duration'];
  }

  $output = '<p>' . t('You are booking an appointment for !product_text. This appointment takes about <strong>@total_duration minutes</strong>.', array('!product_text' => $product_text, '@total_duration' => $total_duration)) . '</p>';

  return $output;
}

/**
 * Helper function for processing the date_of_birth field in step 3.
 */
function _appointments_process_date_of_birth(&$element) {
  $year = appointments_format_date(REQUEST_TIME, 'year');
  $start_year = $year + $element['#start_year'];
  $end_year = $year + $element['#end_year'];
  $element['year']['#options'] = array('0' => t('Year')) + drupal_map_assoc(range($start_year, $end_year));
  $element['month']['#options'] = array('0' => t('Month')) + $element['month']['#options'];
  $element['day']['#options'] = array('0' => t('Day')) + $element['day']['#options'];
  return $element;
}
