<?php

class AppointmentsQmaticClient extends AppointmentsClientApi {

  function __construct() {
    $link = variable_get('webform_appointment_client_api', '');
    if (empty($link)) {
      throw new Exception('No WSDL configured.');
    }

    $this->soap_client = new SoapClient($link);
  }

  public function get_available_products() {
    $products = array();
    $available_products = $this->call('getAvailableProducts');
    foreach ($available_products->getAvailableProductsReturn as $product) {
      $products[$product->linkId] = array(
        'id' => $product->linkId,
        'name' => $product->name,
        'duration' => $product->appointLength,
      );
    }

    uasort($products, array($this, 'sort_available_products'));

    return $products;
  }

  public function get_available_appointment_days($product_id) {
    $available_appointment_days = $this->call('getAvailableAppointmentDays', array(
      'productLinkID' => $product_id,
    ));

    return $available_appointment_days->getAvailableAppointmentDaysReturn;
  }

  public function get_available_appointment_times($product_id, $date) {
    $available_appointment_times = $this->call('getAvailableAppointmentTimes', array(
      'productLinkID' => $product_id,
      'appDate' => $date,
    ));

    return $available_appointment_times->getAvailableAppointmentTimesReturn;
  }

  public function get_customer_appointments($vars) {
    $customer_id = md5($vars['email']);
    $now = new DateTime();
    $now = $now->format('Y-m-d\TH:i:s\Z');
    $get_customer_appointment = $this->call('getCustomerAppointments', array(
      'customerId' => $customer_id,
      'fromDate' => $now,
    ));

    if (isset($get_customer_appointment->getCustomerAppointmentsReturn)) {
      $appointments = $get_customer_appointment->getCustomerAppointmentsReturn;
      if (!is_array($appointments)) {
        $appointments = array($appointments);
      }
      foreach ($appointments as $appointment) {
        if ($appointment->appointmentId == $vars['appointment_id']) {
          return $appointment;
        }
      }
    }

    return FALSE;
  }

  public function delete_appointment($appointment_id) {
    $delete_appointment = $this->call('deleteAppointment', array(
      'appointmentId' => $appointment_id,
    ));

    return $delete_appointment->deleteAppointmentReturn;
  }

  public function book_appointment($appointment_data, $book_data) {
    $date = webform_appointment_format_date($appointment_data['datetime'], 'custom', 'Y-m-d') . 'T00:00:00.000Z';
    $time = '1970-01-01T' . webform_appointment_format_date($appointment_data['datetime'], 'custom', 'H:i:s') . '.000Z';

    if (!isset($book_data['piNo']) && isset($book_data['email'])) {
      $book_data['piNo'] = md5($book_data['email']);
    }
    $book_appointment = $this->call('bookAppointment', array(
      'productLinkID' => (isset($appointment_data['id'])) ? $appointment_data['id'] : '',
      'appDate' => $date,
      'appTime' => $time,
      'name' => (isset($book_data['name'])) ? $book_data['name'] : '',
      'address' => (isset($book_data['address'])) ? $book_data['address'] : '',
      'town' => (isset($book_data['town'])) ? $book_data['town'] : '',
      'state' => (isset($book_data['state'])) ? $book_data['state'] : '',
      'zipCode' => (isset($book_data['zipCode'])) ? $book_data['zipCode'] : '',
      'country' => (isset($book_data['country'])) ? $book_data['country'] : '',
      'phone' => (isset($book_data['phone'])) ? $book_data['phone'] : '',
      'phone2' => (isset($book_data['phone2'])) ? $book_data['phone2'] : '',
      'email' => (isset($book_data['email'])) ? $book_data['email'] : '',
      'piNo' => $book_data['piNo'],
      'accountNo' => (isset($book_data['accountNo'])) ? $book_data['accountNo'] : '',
      'cardNum' => (isset($book_data['cardNum'])) ? $book_data['cardNum'] : '',
      'title' => (isset($book_data['title'])) ? $book_data['title'] : '',
      'born' => (isset($book_data['born'])) ? $book_data['born'] : '',
      'vipLevel' => (isset($book_data['vipLevel'])) ? $book_data['vipLevel'] : '',
      'gender' => (isset($book_data['gender'])) ? $book_data['gender'] : '',
      'social' => (isset($book_data['social'])) ? $book_data['social'] : '',
      'family' => (isset($book_data['family'])) ? $book_data['family'] : '',
      'customerClass' => (isset($book_data['customerClass'])) ? $book_data['customerClass'] : '',
      'comments' => (isset($book_data['comments'])) ? $book_data['comments'] : '',
    ));
    $return = $book_appointment->bookAppointmentReturn;

    // $this->send_email($return, $appointment_data, $book_data); @todo: Cannot use object of type stdClass as array

    return array(
      'id' => $return->appointmentId,
      'status' => $return->appointmentStatus,
    );
  }

  public function get_dates_times($product_ids, $durations, $persons) {
    $dates_all = array();
    foreach ($product_ids as $product_link_id) {
      foreach ($this->get_available_appointment_days($product_link_id) as $date) {
        $time = $this->get_available_appointment_times($product_link_id, $date);
        if (!isset($dates_all[$product_link_id])) {
          $dates_all[$product_link_id] = array();
          $dates_all[$product_link_id]['dates'] = array();
          $dates_all[$product_link_id]['product_id'] = $product_link_id;
          $dates_all[$product_link_id]['duration'] = $durations[$product_link_id];
        }
        if (!isset($dates_all[$product_link_id]['dates'][$date])) {
          $dates_all[$product_link_id]['dates'][$date] = is_array($time) ? $time : array($time);
        }
      }
    }

    // _appointments_magic checks the dates.
    $right_dates_time = $this->_appointments_magic($dates_all, $persons);

    return $right_dates_time;
  }

    /**
     * Magic function that check all correct time for appointments.
     */
  protected function _appointments_magic($dates_all, $persons = array()) {
    $available_dates = $this->check_dates($dates_all);

    $good_times = array();
    $dates = array_shift($dates_all);
    foreach ($available_dates as $date) {
      foreach ($dates['dates'][$date] as $time) {
        $person_is_ok = array();
        $appointments = array();
        for ($i = 0; $i < $persons[$dates['product_id']]; $i++) {
          // Why was ($i+1) ? Deleted because 9;00 was not visible
          $search_time = strtotime($time) + $dates['duration'] * 60 * ($i);
          foreach ($dates['dates'][$date] as $_time) {
            if ($search_time == strtotime($_time)) {
              $person_is_ok[$i] = TRUE;
              $appointments[$_time] = array(
                'product_id' => $dates['product_id'],
                'duration' => $dates['duration'],
                'date' => $date,
                'time' => $_time,
              );
            }
          }
        }

        // Is this ok? Then check for person 1
        $is_ok = TRUE;
        for ($p = 0; $p < $persons[$dates['product_id']]; $p++) {
          if (!isset($person_is_ok[$p])) {
            $is_ok = FALSE;
          }
        }

        if ($is_ok) {
          $good_times[] = array_values($appointments);
        }
      }
    }

    foreach ($dates_all as $dates) {
      $good_times_temp = $good_times;
      $good_times = array();
      foreach ($available_dates as $date) {
        foreach ($dates['dates'][$date] as $time) {
          foreach ($good_times_temp as $appointments) {
            $last_appointment = end($appointments);

            if ($date == $last_appointment['date'] && (strtotime($time) == strtotime($last_appointment['time']) + $last_appointment['duration'] * 60)) {
              $person_is_ok = array();
              for ($i = 1; $i <= $persons[$dates['product_id']]; $i++) {
                // This part should check the next moment that is available. The for-loop should start.
                $search_time = strtotime($last_appointment['time']) + $last_appointment['duration'] * 60 * ($i);
                foreach ($dates['dates'][$date] as $_time) {
                  if ($search_time == strtotime($_time)) {
                    $person_is_ok[$i] = TRUE;
                    $appointments[] = array(
                      'product_id' => $dates['product_id'],
                      'duration' => $dates['duration'],
                      'date' => $date,
                      'time' => $_time,
                    );
                  }
                }
              }

              // Is this OK? Then start with 1 because that should be added ($i) to the time in the above loop to find the next moment.
              $is_ok = TRUE;
              for ($o = 1; $o <= $persons[$dates['product_id']]; $o++) {
                if (!isset($person_is_ok[$o])) {
                  $is_ok = FALSE;
                }
              }

              if ($is_ok) {
                $good_times[] = $appointments;
              }
            }
          }
        }
      }
    }

    // Returns all good times that need to be selectable.
    return $good_times;
  }

  /**
   * Helper function which check the same date for appointments.
   */
  protected function check_dates($dates_all) {
    $right_dates = array();

    $first_dates = array_shift($dates_all);
    foreach ($first_dates['dates'] as $date => $time) {
      $right_dates[] = $date;
    }

    foreach ($dates_all as $product) {
      $right_dates_temp = $right_dates;
      $right_dates = array();
      foreach ($product['dates'] as $date => $time) {
        if (in_array($date, $right_dates_temp)) {
          $right_dates[] = $date;
        }
      }
    }

    return $right_dates;
  }

  public function get_book_fields() {
    return array(
      'name' => 'Name',
      'phone' => 'Telephone',
      'email' => 'Email',
      'born' => 'Birthdate',
    );
  }

  protected  function send_email($appointment, $appointment_data, $book_data) {
    $mail_array = array(
      '%id' => trim($appointment['id']),
      '%subject' => $appointment_data['name'],
      '%estimated_duration' => $appointment_data['duration'],
      '%date' => appointments_format_date($appointment_data['datetime'], 'dvg_short_date'),
      '%start_time' => appointments_format_date($appointment_data['datetime'], 'time'),
      '%end_time' => appointments_format_date($appointment_data['datetime'] + ($appointment_data['duration'] * 60), 'time'),
      '%first_name' => '',
      '%surname' => isset($book_data['name']) ? $book_data['name'] : '',
      '%date_of_birth' => $book_data['born'] ? appointments_format_date(strtotime($book_data['born']), 'date') : '',
      '%email' => $book_data['email'],
      '%telephone_number' => $book_data['phone'],
      '%bsn' => isset($book_data['bsn']) ? $book_data['bsn'] : '',
      // @todo cancel link
      '%cancel_appointment_link' => url('annuleren-afspraak', array('absolute' => TRUE)),
      '%task_confirm_text' => $appointment_data['confirm_text'],
      '%field_confirm_appointment' => '',
    );

    $keys = array_keys($mail_array);
    $params = array(
      'subject' => str_replace($keys, $mail_array, variable_get('appointments_confirmation_mail_title', '')),
      'message' => str_replace($keys, $mail_array, variable_get('appointments_confirmation_mail_text', '')),
    );

    drupal_mail('qmatic_api', 'confirmation', $book_data['email'], 'en', $params);

    $second_email = variable_get('appointments_second_email', '');
    if (!empty($second_email)) {
      drupal_mail('webform_appointment', 'confirmation', $second_email, 'en', $params);
    }
  }
}
