<?php

abstract class AppointmentsClientApi {

  protected $soap_client;

  abstract function __construct();

  protected function call($function, $args = array(), $reset = FALSE) {
    try {
      $start = microtime(TRUE);

      ksort($args);
      $cid = 'webform_appointment__' . $function . '__' . substr(md5(serialize($args)), 0, 8);
      if (!$reset && $cache = cache_get($cid)) {

        // Make sure the cache isn't expired.
        if ($cache->expire > REQUEST_TIME) {
          return $cache->data;
        }
      }

      $return = $this->soap_client->{$function}($args);

      $duration = microtime(TRUE) - $start;
      if ($duration >= variable_get('webform_appointment_api_duration_threshold', 2)) {
        watchdog('webform_appointment', '@method duration: @durations (@info)', array(
          '@method' => $function,
          '@duration' => round($duration, 2),
          '@info' => var_export($args, TRUE),
        ), WATCHDOG_WARNING);
      }

      $custom_expire = variable_get('webform_appointment__cache__' . $function, FALSE);
      if ($custom_expire || $custom_expire === FALSE) {
        $expire = REQUEST_TIME + ($custom_expire ? $custom_expire : variable_get('webform_appointment__cache_default', 3600));
        cache_set($cid, $return, 'cache', $expire);
      }

      return $return;
    }
    catch (SoapFault $f) {
      watchdog('appointment_api', 'Soap Fault: ' . var_export($f, TRUE), WATCHDOG_ERROR);
      return FALSE;
    }
  }

  protected function sort_available_products($a, $b) {
    return strcmp($a['name'], $b['name']);
  }

  abstract public function get_book_fields();

  public function set_booking_value($field, $value) {
    return $value;
  }

  abstract public function get_available_products();

  abstract public function get_available_appointment_days($product_id);

  abstract public function get_available_appointment_times($product_id, $date);

  abstract public function get_customer_appointments($vars);

  abstract public function delete_appointment($appointment_id);

  abstract public function book_appointment($appointment_data, $book_data);

  abstract public function get_dates_times($product_ids, $durations, $persons);
}
